;Name:      BSCHRAFF.LSP
;Version:   1.01
;Autor:     Markus Guthmann
;Datum:     21.6.93
;Sprache:   AutoLisp R12
;Bemerkung: GEOMCAL.EXP mu verfgbar sein
;Zweck:     Programm generiert eine Bschungsschraffur zwischen
;           zwei Polylinien.
;Aufruf:    1. BSCHRAFF
;           2. Schraffurabstand eingeben
;           3. danach obere und untere Bschungskante (Polylinie) picken
;Revision:  20.1.1994 Fehler bei konvexen B"schungsschraffuren behoben
;***********************************************************************
(defun GetPLineIntersec(pa pb pname / plist ptype pmode subname sublist sub-etype p0 p1 px f70 first l pl)
(setvar "osmode" 0)
;berechnet den Schnittpunkt der durch pa und pb gegebenen Geraden mit der 
;Polylinie pname  
;Rckgabewert ist, falls vorhanden, der Schnittpunkt, andernfalls nil
  ;den Polylinientyp ermitteln
  (setq plist (entget pname))
  (setq ptype (cdr (assoc 70 plist)))
  (if (= (logand ptype 4) 4)
    ;Polylinie ist ein Spline
    (setq pmode 8)
    (setq pmode 0)
  )

  ;der Elementname der auf den Polyliniennamen folgt, 
  ;ist der Name der ersten Scheitelpunktliste
  (setq subname (entnext pname))
  
  ;Scheitelpunktliste in sublist bertragen 
  (setq sublist (entget subname))       
																  
  ;Punktflag ermitteln
  (setq f70 (cdr (assoc 70 sublist)))

  ; der Startpunkt 
  (setq p0 (cdr (assoc 10 sublist)))

  (setq first 1)
  (setq pl nil)
 
  ;solange Scheitelpunkte gefunden werden
  (while (and (eq (setq sub-etype (cdr (assoc 0 sublist))) "VERTEX") (/= break 1))
				
	;nur nach Splinepunkten suchen
	(setq f70 (cdr (assoc 70 sublist)))
	(if (= (logand f70 pmode) pmode) 
	  (progn

		;der zweite Punkt  
		(setq p1 (cdr (assoc 10 sublist))) 
	  
		;gleiche Punkte ausfiltern
		(if (> (distance p0 p1) 0.0001)

		  (progn
			
			;Schnittpunkt mit dem Segment berechnen
			(cal "px=ill(pa, pb, p0, p1)")

			;liegt der Schnittpunkt im Segment ?
			(setq l (distance p0 p1))
			(if (and (<= (distance p0 px) l) (<= (distance p1 px) l))
			  (progn
			    (if (= first 1) (setq pl px))
			    (setq first 0)                        
			    (if (< (distance pa px) (distance pa pl))
			      (setq pl px)
			    )
			  )
			)

		  ) ;progn
			  
		) ;if

		;alter Endpunkt ist neuer Startpunkt
		(setq p0 p1)  

	  ) ;progn

	)
	
	;nchster Scheitelpunktname
	(setq subname (entnext subname))    
	;Scheitelpunkt --> sublist
	(setq sublist (entget subname))      

  )

  (setq pl pl)

)

(defun DividePLine(pname0 pname1 dl / plist ptype pmode elist sublist subname sub-etype p0 p1 px py ps nv len i r d f70)
;teilt die angegebene Polylinie in Abschnitte von dl-Lnge

  ;den Polylinientyp ermitteln
  (setq plist (entget pname0))
  (setq ptype (cdr (assoc 70 plist)))
  (if (= (logand ptype 4) 4)
    ;Polylinie ist ein Spline
    (setq pmode 8)
    (setq pmode 0)
  )

  ;der Elementname der auf den Polyliniennamen folgt, 
  ;ist der Name der ersten Scheitelpunktliste
  (setq subname (entnext pname0))
  
  ;Scheitelpunktliste in sublist bertragen 
  (setq sublist (entget subname))       
																  
  (setq len 0)
  (setq i 0)

  ;Punktflag ermitteln
  (setq f70 (cdr (assoc 70 sublist)))

  ; der Startpunkt 
  (setq p0 (cdr (assoc 10 sublist)))
 
  ;solange Scheitelpunkte gefunden werden
  (while (eq (setq sub-etype (cdr (assoc 0 sublist))) "VERTEX")
				
	;nur nach Splinepunkten suchen
	(setq f70 (cdr (assoc 70 sublist)))
	(if (= (logand f70 pmode) pmode) 
	  (progn

		;der zweite Punkt  
		(setq p1 (cdr (assoc 10 sublist))) 
	  
		;gleiche Punkte ausfiltern
		(if (> (distance p0 p1) 0.0001)

		  (progn

			;Abstand des Punktes von p0 aus berechnen
			(setq r (* i dl))
			(setq d (- r len))    

			;liegt der Punkt berhaupt im Segment ?
			(if (> (distance p0 p1) d)

			  ;Punkt liegt im Segment
			  (progn

				;Punkt berechnen
				(cal "px=pld(p0, p1, d)")

				;Normalenvektor berechnen
				(cal "nv=nor(p0, p1)")
				(cal "py=px+nv")

				;Schnittpunkt mit der zweiten Polylinie berechnen
				(setq ps (GetPLineIntersec px py pname1))
				(if (/= ps nil)
				  (progn
					(if (= (rem i 2) 1)
					  (cal "ps=plt(px, ps, 0.5)")
					)
					(command "_line" px ps "")
				  )
				)

				;Index des nchsten Punkts setzen
				(setq i (+ i 1))

				;noch Platz fr weitere Punkte im Segment ?   
				(while (> (distance px p1) dl)
				  
				  ;Abstand des weiteren Punktes von p0 aus berechnen
				  (setq r (* i dl))
				  (setq d (- r len))    
				  ;weiteren Punkt berechnen
				  (cal "px=pld(p0, p1, d)")
				
				  ;Normalenvektor berechnen
				  (cal "nv=nor(p0, p1)")
				  (cal "py=px+nv")

				  ;Schnittpunkt mit der zweiten Polylinie berechnen
				  (setq ps (GetPLineIntersec px py pname1))
				  (if (/= ps nil)
					(progn 
					  (if (= (rem i 2) 1)
						(cal "ps=plt(px, ps, 0.5)")
					  )
					  (command "_line" px ps "")
					)
				  )
				  
				  ;Index des nchsten Punkts setzen
				  (setq i (+ i 1))                  
				  
				) ;while

			  ) ;progn
			
			) ;if

			;Polylinienl"nge berechnen
			(setq len (+ len (distance p0 p1))) 

		  ) ;progn
			  
		) ;if

		;alter Endpunkt ist neuer Startpunkt
		(setq p0 p1)  

	  ) ;progn

	)
	
	;nchster Scheitelpunktname
	(setq subname (entnext subname))    
	;Scheitelpunkt --> sublist
	(setq sublist (entget subname))      

  )

)

(defun GetPolyline(/ ename etype elist break)
;Prozedur veranlat den Bediener eine Polylinie zu picken
;Rckgabewert ist der Elementname einer Polylinie  
  
  (setq break 0)
  (while (/= break 1)

	;den Namen (Offset 0 der Auswahlliste) eines gewhlten Objekts holen
	(setq ename (car (entsel "")))
	(if (/= ename nil)
	  (progn

		;die Hauptliste in elist bertragen
		(setq elist (entget ename))
		;den Typnamen ermitteln (Gruppencode 0)
		(setq etype (cdr (assoc 0 elist)))

		(if (= etype "POLYLINE")
		  (setq break 1)
		  (print "gewaehltes Objekt ist keine Polylinie!")
		)
	  
	  )
	  (print "kein Objekt gewaehlt!")
	)
  )
  ;Rckgabewert sicherstellen
  (setq ename ename)
)

(defun c:bschraff(/ old_cmdecho sdist polyl0 polyl1)
;Schnittstellenprozedur
  (setq osmodeold (getvar "OSMODE"))
  (setq old_cmdecho (getvar "CMDECHO")) 
  (setvar "CMDECHO" 0)

  ;Geometrierechner laden 
  ;(xload "geomcal")

  (setq sdist (getreal "Schraffurabstand: "))
  (princ "\nObere Bschungskante picken:  ")
  (setq polyl0 (GetPolyline))
  (princ "\nUntere Bschungskante picken: ")
  (setq polyl1 (GetPolyline))
  (DividePLine polyl0 polyl1 sdist)

    (setvar "CMDECHO" old_cmdecho)
    (setvar "osmode" 8)   
    (setvar "osmode" osmodeold)

)


